local addonName, L = ...;

function ArenaLive_FrameMover_OnLoad (self, affectedAddonName, affectedframeType, frameToMove)
	
	if ( not self or not affectedAddonName or not affectedframeType or not frameToMove ) then
		return;
	end
	
	local prefix = self:GetName();
	local affectedWidth = frameToMove:GetWidth();
	local selfWidth = self:GetWidth();
	
	if ( selfWidth < affectedWidth ) then
		self:SetWidth(affectedWidth);
	end
	
	self.frameToMove = frameToMove;
	self.addonName = affectedAddonName;
	self.frameType = affectedframeType;
	self.text = _G[prefix.."Text"];
	local left, bottom, width, height = self:GetRect();
	self.text:SetText(string.format(L["FrameMoverText"], prefix, left, bottom));
	
end

function ArenaLive_FrameMover_StartMoving (self, button)
	local isLocked = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["isLocked"];
	if ( not isLocked and button == "LeftButton" ) then
		self.frameToMove:StartMoving();
		self.isMoving = true;
		self.frameToMove:SetClampedToScreen(true);
	end
end

function ArenaLive_FrameMover_StopMoving (self)
	if ( self.isMoving ) then	
		self.frameToMove:StopMovingOrSizing();
		self.isMoving = false;
		
		local point, relativeTo, relativePoint, xOffset, yOffset = self.frameToMove:GetPoint();
		
		ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["anchor"]["point"] = point;
		ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["anchor"]["relativeTo"] = relativeTo;
		ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["anchor"]["relativePoint"] = relativePoint;
		ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["anchor"]["xOffset"] = xOffset;
		ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["anchor"]["yOffset"] = yOffset;
		
		ArenaLive_Database_UpdateSavedVariables(self.addonName, ArenaLiveUnitFrames_SavedVariables);
	end
end

function ArenaLive_FrameMover_OnUpdate (self, elapsed)
	local frameName = self.frameToMove:GetName();
	local left, bottom, width, height = self:GetRect();
	self.text:SetText(string.format(L["FrameMoverText"], frameName, left, bottom));	

end

function ArenaLive_FrameMover_UpdateVisible (self)
	if ( not self ) then
		return;
	end
	local isLocked = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["isLocked"];
	
	if ( isLocked ) then
		self:Hide();
		if ( self.frameToMove.registerUnitWatch ) then
			RegisterUnitWatch(self.frameToMove);
		end
	else
		self:Show();
		if ( self.frameToMove.registerUnitWatch ) then
			UnregisterUnitWatch(self.frameToMove);
		end
		self.frameToMove:Show();
	end

end

function ArenaLive_FrameMover_OnShow (self)
	self:SetScript("OnUpdate", ArenaLive_FrameMover_OnUpdate);
end

function ArenaLive_FrameMover_OnHide (self)
	self:SetScript("OnUpdate", nil);
	ArenaLive_FrameMover_StopMoving(self);
end