local maxBuffs = 40;
local maxDebuffs = 40;

function ArenaLive_PortraitOverlay_Initialise (overlay, parent, portrait)
	
	if ( not overlay ) then
		return;
	end
	
	local prefix = overlay:GetName();
	local portraitWidth, portraitHeight = portrait:GetSize();
	local point, relativeTo, relativePoint, xOffset, yOffset = portrait:GetPoint();
	
	overlay.spellPriority = 0;
	overlay.duration = 0;
	overlay:SetSize(portraitWidth, portraitHeight);
	overlay:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset);
	overlay.parent = parent;
	overlay.icon = _G[prefix.."Icon"];
	ArenaLive_Cooldown_Initialise(_G[prefix.."Cooldown"], overlay);

end



function ArenaLive_PortraitOverlay_QueryAuras (parent, unit)

	-- Before we query the auras, we need to check if our current shown spell has run out or was changed somehow
	if ( parent.portraitOverlay.auraID ) then
		
		local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID;
		
		if ( parent.portraitOverlay.auraType == "Buff" ) then
			name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID = UnitBuff(parent.unit, parent.portraitOverlay.auraID)
		else
			name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID = UnitDebuff(parent.unit, parent.portraitOverlay.auraID)
		end
		
		if (spellID ~= parent.portraitOverlay.spellID ) then
			parent.portraitOverlay.icon:SetTexture();
			parent.portraitOverlay.spellID = nil;
			parent.portraitOverlay.auraID = nil;
			parent.portraitOverlay.auraType = nil;
			parent.portraitOverlay.duration = 0;
			parent.portraitOverlay.spellPriority = 0;
			
			if ( parent.portraitOverlay.cooldown ) then
				ArenaLive_Cooldown_Set(parent.portraitOverlay.cooldown, GetTime(), 0)
			end
		end
		
	end
	
	for i = 1, maxBuffs do
	
		local name, _, icon, _, dispelType, duration, expirationTime, _, _, _, spellID = UnitBuff(unit, i)
		
		if (name) then
			ArenaLive_PortraitOverlay_Update(parent, spellID, duration, expirationTime, i, "Buff")
		else
			-- There are no auras left, so we can stop here
			break;
		end
	end

	for i = 1, maxDebuffs do
	
		local name, _, icon, _, dispelType, duration, expirationTime, _, _, _, spellID = UnitDebuff(unit, i)
		
		if (name) then
			ArenaLive_PortraitOverlay_Update(parent, spellID, duration, expirationTime, i, "Debuff")
		else
			-- There are no auras left, so we can stop here
			break;
		end
	end	
	
end



function ArenaLive_PortraitOverlay_OnUpdate (self, elapsed)

	if ( self.spellID ) then
		local name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID;

		self.duration = self.duration - elapsed;
	
		if ( self.auraType == "Buff" ) then
			name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID = UnitBuff(self.parent.unit, self.auraID)
		else
			name, rank, icon, count, dispelType, duration, expires, caster, isStealable, shouldConsolidate, spellID = UnitDebuff(self.parent.unit, self.auraID)
		end

		if ( self.duration <= 0 ) then
			self.spellID = nil;
			self.auraID = nil;
			self.auraType = nil;
			self.duration = 0;
			self.spellPriority = 0;
			self.icon:SetTexture();
			if ( self.cooldown ) then
				ArenaLive_Cooldown_Set( self.cooldown, GetTime(), 0)
			end
		end
		
	end

end



function ArenaLive_PortraitOverlay_Update (unitFrame, spellID, auraDuration, expirationTime, auraID, auraType)
	
	-- First check, if the spellID is one we need and if we have an overlay.
	if ( ( not ARENALIVE_SPELLS["portraitOverlay"][spellID] ) or ( not unitFrame.portraitOverlay ) ) then
		return;
	end
	
	local ccType = ARENALIVE_SPELLS["portraitOverlay"][spellID];
	local spellPriority = ARENALIVE_DATABASE["options"]["overlayPriorities"][ccType];
	
	-- If spellPriority is 0 that means that the spell type is deactivated
	if ( spellPriority == 0 ) then
		return;
	end
	
	-- TO DO: Values for SmokeBomb, SolarBeam etc.
	
	local startTime = expirationTime - auraDuration;
	local duration = expirationTime - GetTime(); -- auraDuration is the full duration of the buff, this one is the real time remaining.
	-- If spellID and expirationTime are different or the auraID, we need to update.
	if ( ( expirationTime ~= unitFrame.portraitOverlay.expirationTime ) or ( auraID ~= unitFrame.portraitOverlay.auraID ) or ( spellID ~= unitFrame.portraitOverlay.spellID ) ) then
	
		-- Now we need to check, if the new aura we've found is more important than the current one.
		if ( ( spellPriority > unitFrame.portraitOverlay.spellPriority ) or ( ( spellPriority == unitFrame.portraitOverlay.spellPriority ) and ( duration > unitFrame.portraitOverlay.duration ) ) ) then
		
			local name, _, icon = GetSpellInfo(spellID)
			unitFrame.portraitOverlay.spellID = spellID;
			unitFrame.portraitOverlay.spellPriority = spellPriority;
			unitFrame.portraitOverlay.duration = duration;
			unitFrame.portraitOverlay.expirationTime = expirationTime;
			unitFrame.portraitOverlay.auraID = auraID;
			unitFrame.portraitOverlay.auraType = auraType;
			
			unitFrame.portraitOverlay.icon:SetTexture(icon);	
			ArenaLive_Cooldown_Set(unitFrame.portraitOverlay.cooldown, startTime, auraDuration)
		end
	end
	
end