local addonName, L = ...;

function ALUF_PetFrame_OnLoad (self)
	
	ArenaLive_SecureUnitButton_OnLoad(self, nil, nil, true);
	
	ArenaLive_UnitFrame_Initialise(self, addonName, "petFrame", ALUF_PetFrameBackground, ALUF_PetFrameBorder, ALUF_PetFrameName, ALUF_PetFramePortrait, ALUF_PetFrame3DPortrait, ALUF_PetFramePortraitOverlay, ALUF_PetFrameHealthBar, ALUF_PetFrameHealthBarText, ALUF_PetFramePowerBar, ALUF_PetFramePowerBarText, nil, nil, ALUF_PetFrameHealPredictionBar, ALUF_PetFrameAbsorbBar, ALUF_PetFrameAbsorbBarOverlay, ALUF_PetFrameAbsorbFullHPIndicator);
	ArenaLive_UnitFrame_UpdateUnit(self, "pet");
	ArenaLive_CastBar_Initialise(ALUF_PetFrameCastBar, addonName, "petFrame", ALUF_PetFrameCastBarBorder, ALUF_PetFrameBorderShieldGlow, ALUF_PetFrameCastBarIcon, ALUF_PetFrameCastBarText, ALUF_PetFrameCastBarAnimationGroup, ALUF_PetFrameCastBarAnimationGroupFadeOutAnimation, false, false);
	ArenaLive_CastBar_UpdateUnit(ALUF_PetFrameCastBar, "pet");
	ArenaLive_AuraFrame_Initialise(ALUF_PetFrameAuras, addonName, "petFrame", ALUF_PetFrameAurasBuffFrame, ALUF_PetFrameAurasDebuffFrame, "ArenaLive_BuffTemplate", "ArenaLive_DebuffTemplate");
	ArenaLive_AuraFrame_UpdateUnit(ALUF_PetFrameAuras, "pet");
	
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("UNIT_PET");
	self:RegisterEvent("PET_ATTACK_START");
	self:RegisterEvent("PET_ATTACK_STOP");
	self:RegisterEvent("PET_UI_UPDATE");
	self:RegisterEvent("PET_RENAMEABLE");
	self:RegisterEvent("UNIT_NAME_UPDATE");
	
	ALUF_PetFrame_AdjustAttachments(self)
	
end

function ALUF_PetFrame_OnEvent(self, event, ...)

	ArenaLive_UnitFrame_OnEvent(self, event, ...)
	
	local arg1, arg2, arg3, arg4, arg5 = ...;
	if ( event == "ADDON_LOADED" ) then
		ALUF_PetFrame_AdjustAttachments(self);
	elseif ( (event == "UNIT_PET" and arg1 == "player" ) or event == "PET_UI_UPDATE" ) then
		local unit
		if ( UnitInVehicle("player") ) then
			if ( UnitHasVehiclePlayerFrameUI("player") ) then
				unit = "player";
			else
				return;
			end
		else
			unit = "pet";
		end
		ArenaLive_UnitFrame_UpdateUnit(self, unit, PetFrameHealthBar, PetFrameManaBar);
		ArenaLive_UnitFrame_Update(self);
		CloseDropDownMenus();
	elseif ( event == "PET_ATTACK_START" ) then
	elseif ( event == "PET_ATTACK_STOP" ) then
	elseif ( event == "PET_RENAMEABLE" ) then
		StaticPopup_Show("RENAME_PET");
	elseif ( event == "UNIT_NAME_UPDATE" ) then
		ArenaLive_UnitFrame_Update(self);
	end
end

function ALUF_PetFrame_AdjustAttachments(self)

	local castBarIsShown, castBarPosition, auraFrameIsShown, auraFramePosition;
	local isEnabled = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["enabled"];
	
	-- First check if the Frame is enabled. If not we can skip the rest.
	ArenaLive_ToggleBlizzFrame(PetFrame, isEnabled);
	
	if ( isEnabled ) then
		ArenaLive_UnitFrame_UpdateUnit(self, "pet");
		PetFrame:SetScript("OnEvent", nil);
	else
		ArenaLive_UnitFrame_ResetFrame(self);
		ArenaLive_CastBar_ResetFrame(ALUF_PetFrameCastBar);
		ArenaLive_AuraFrame_ResetFrame(ALUF_PetFrameAuras);
		self:Hide();
		PetFrame:SetScript("OnEvent", PetFrame_OnEvent);
		return;
	end	
	
	if ( self.castBar ) then
		castBarIsShown = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["petFrame"]["castBar"]["isShown"];
		castBarPosition = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["petFrame"]["castBar"]["position"];
	end
	
	if ( self.auraFrame ) then
		auraFrameIsShown = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["petFrame"]["auraFrame"]["isShown"];
		auraFramePosition = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["petFrame"]["auraFrame"]["position"];
	end

	if ( castBarIsShown ) then
		ArenaLive_CastBar_UpdateUnit(ALUF_PetFrameCastBar, "pet");
		self.castBar:ClearAllPoints();
		
		if ( castBarPosition == auraFramePosition and auraFrameIsShown ) then
			--[[ castBar and auraFrame are either both above or below the unitframe. In this case we must anchor the castbar
			to the auraFrame, so that it will change it's position according to the height of the auraFrame. ]]--
			
			if ( castBarPosition == "BELOW" ) then
				self.castBar:SetPoint("TOPLEFT", self.auraFrame, "BOTTOMLEFT", 0, -3)
			elseif ( castBarPosition == "ABOVE" ) then
				self.castBar:SetPoint("BOTTOMLEFT", self.auraFrame, "TOPLEFT", 0, 3)
			end
		else
			--[[ Castbar and  auraFrame are on different spots, so the castbar needs to be anchored to the unitFrame itself]]--
			
			if ( castBarPosition == "BELOW" ) then
				self.castBar:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", -7, 3)
			elseif ( castBarPosition == "ABOVE" ) then
				self.castBar:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", -7, 3)
			end
		end
	else
		ArenaLive_CastBar_ResetFrame(ALUF_PetFrameCastBar);
	end	
	
	if ( auraFrameIsShown ) then
		self.auraFrame:ClearAllPoints();
		
		if ( auraFramePosition == "BELOW" ) then
			self.auraFrame:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, 3);
			ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["petFrame"]["auraFrame"]["growUpwards"] = false;
		elseif ( auraFramePosition == "ABOVE" ) then
			self.auraFrame:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 2);
			ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["petFrame"]["auraFrame"]["growUpwards"] = true;
		end
		
		ArenaLive_AuraFrame_UpdateUnit(ALUF_PetFrameAuras, "pet");
	else
		ArenaLive_AuraFrame_ResetFrame(ALUF_PetFrameAuras)
	end
end