local addonName, L = ...;

function ALUF_TargetFrame_OnLoad (self)
	
	ArenaLive_SecureUnitButton_OnLoad(self, nil, nil, true);
	
	local numCastHistoryIcons = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["castHistory"]["shownIcons"];
	local castHistoryIconSize = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["castHistory"]["iconSize"];
	
	self.showPVP = true;
	self.showLevel = true;
	self.showClassification = true;
	self.showLeader = true;
	
	self.flash = ALUF_TargetFrameFlash;
	self.levelText = ALUF_TargetFrameLevelText;
	self.highLevelTexture = ALUF_TargetFrameHighLevelTexture;
	self.PVPIcon = ALUF_TargetFramePVPIcon;
	self.raidTargetIcon = ALUF_TargetFrameRaidTargetIcon;
	self.leaderIcon = ALUF_TargetFrameLeaderIcon;
	self.questIcon = ALUF_TargetFrameQuestIcon;
	self.petBattleIcon = ALUF_TargetFramePetBattleIcon;
	
	ArenaLive_UnitFrame_Initialise(self, "ArenaLiveUnitFrames", "targetFrame", ALUF_TargetFrameBackground, ALUF_TargetFrameBorder, ALUF_TargetFrameName, ALUF_TargetFramePortrait, ALUF_TargetFrame3DPortrait, ALUF_TargetFramePortraitOverlay, ALUF_TargetFrameHealthBar, ALUF_TargetFrameHealthBarText, ALUF_TargetFramePowerBar, ALUF_TargetFramePowerBarText, self.flash, "player", ALUF_TargetFrameHealPredictionBar, ALUF_TargetFrameAbsorbBar, ALUF_TargetFrameAbsorbBarOverlay, ALUF_TargetFrameAbsorbFullHPIndicator);
	ArenaLive_UnitFrame_UpdateUnit(self, "target");
	ArenaLive_Icon_Initialise(ALUF_TargetFrameIcon1, "ArenaLiveUnitFrames", "targetFrame", 1, ALUF_TargetFrameIcon1Texture, ALUF_TargetFrameIcon1Cooldown);
	ArenaLive_Icon_UpdateUnit(ALUF_TargetFrameIcon1, "target");
	ArenaLive_Icon_Initialise(ALUF_TargetFrameIcon2, "ArenaLiveUnitFrames", "targetFrame", 2, ALUF_TargetFrameIcon2Texture, ALUF_TargetFrameIcon2Cooldown);
	ArenaLive_Icon_UpdateUnit(ALUF_TargetFrameIcon2, "target");
	ArenaLive_CastBar_Initialise (ALUF_TargetFrameCastBar, "ArenaLiveUnitFrames", "targetFrame", ALUF_TargetFrameCastBarBorder, ALUF_TargetFrameCastBarBorderShieldGlow, ALUF_TargetFrameCastBarIcon, ALUF_TargetFrameCastBarText, ALUF_TargetFrameCastBarAnimationGroup, ALUF_TargetFrameCastBarAnimationGroupFadeOutAnimation, false, true);
	ArenaLive_CastHistory_Initialise(ALUF_TargetFrameCastHistory, "ArenaLiveUnitFrames", "targetFrame", numCastHistoryIcons, "ArenaLive_CastHistoryIconTemplate", "left");
	ArenaLive_AuraFrame_Initialise(ALUF_TargetFrameAuras, "ArenaLiveUnitFrames", "targetFrame", ALUF_TargetFrameAurasBuffFrame, ALUF_TargetFrameAurasDebuffFrame, "ArenaLive_BuffTemplate", "ArenaLive_DebuffTemplate");	
	
	ALUF_TargetFrame_AdjustAttachments(self);
	ALUF_TargetFrame_Update(self);
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("UNIT_CLASSIFICATION_CHANGED");
	self:RegisterEvent("PLAYER_FLAGS_CHANGED");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("RAID_TARGET_UPDATE");
	self:RegisterEvent("UNIT_TARGETABLE_CHANGED");
	self:RegisterEvent("UNIT_LEVEL");
end

function ALUF_TargetFrame_OnEvent(self, event, ...)
	
	if ( not self.unit ) then
		return;
	end
	
	ArenaLive_UnitFrame_OnEvent(self, event, ...);
	
	local arg1 = ...;
	if ( event == "PLAYER_ENTERING_WORLD" ) then		
		ALUF_TargetFrame_Update(self);
		ALUF_TargetFrame_AdjustAttachments(self);
	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
		ALUF_TargetFrame_Update(self);
		TargetFrame_UpdateRaidTargetIcon(self);
		CloseDropDownMenus();
	elseif ( event == "UNIT_TARGETABLE_CHANGED" and arg1 == self.unit) then
		ALUF_TargetFrame_Update(self);
		TargetFrame_UpdateRaidTargetIcon(self);
	elseif ( event == "UNIT_LEVEL" ) then
		if ( arg1 == self.unit ) then
			TargetFrame_CheckLevel(self);
		end
	elseif ( event == "UNIT_FACTION" ) then
		if ( arg1 == self.unit or arg1 == "player" ) then
			ALUF_TargetFrame_CheckFaction(self);
			if ( self.showLevel ) then
				TargetFrame_CheckLevel(self);
			end
		end
	elseif ( event == "UNIT_CLASSIFICATION_CHANGED" ) then
		if ( arg1 == self.unit ) then
			ALUF_TargetFrame_CheckClassification(self);
		end
	elseif ( event == "PLAYER_FLAGS_CHANGED" ) then
		if ( arg1 == self.unit ) then
			if ( UnitLeadsAnyGroup(self.unit) ) then
				self.leaderIcon:Show();
			else
				self.leaderIcon:Hide();
			end
		end
	elseif ( event == "GROUP_ROSTER_UPDATE" ) then
			ALUF_TargetFrame_CheckFaction(self);
	elseif ( event == "RAID_TARGET_UPDATE" ) then
		TargetFrame_UpdateRaidTargetIcon(self);
	end
end


function ALUF_TargetFrame_Update(self)
	
	if ( UnitExists(self.unit) ) then

		-- We must set the unitGUID every time the frame is updated, because of the castHistory. Also reset Icons.
		self.castHistory.guid = UnitGUID("target");
		ArenaLive_CastHistory_ResetAllIcons(self.castHistory);
		
		-- We use the PLAYER_ENTERING_WORLD event to update the castbar.
		ArenaLive_CastBar_OnEvent(self.castBar, "PLAYER_ENTERING_WORLD", nil);
		ArenaLive_UnitFrame_Update(self);
		ArenaLive_AuraFrame_UpdateAuras(self.auraFrame);
		ArenaLive_Icon_Update(self.icon1);
		ArenaLive_Icon_Update(self.icon2);
		ArenaLive_PortraitOverlay_QueryAuras(self, self.unit);
		if ( self.showLevel ) then
			-- We can use the blizzard function here, because It doesn't need to be altered to work with our frames.
			TargetFrame_CheckLevel(self);
		end
		
		ALUF_TargetFrame_CheckFaction(self);
		
		if ( self.showClassification ) then
			ALUF_TargetFrame_CheckClassification(self);
		end
		
		if ( self.showLeader ) then
			if ( UnitLeadsAnyGroup(self.unit) ) then
				if ( HasLFGRestrictions() ) then
					self.leaderIcon:SetTexture("Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES");
					self.leaderIcon:SetTexCoord(0, 0.296875, 0.015625, 0.3125);
				else
					self.leaderIcon:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon");
					self.leaderIcon:SetTexCoord(0, 1, 0, 1);
				end
				self.leaderIcon:Show();
			else
				self.leaderIcon:Hide();
			end
		end
		
		-- We can use the blizzard function here, because It doesn't need to be altered to work with our frames.
		TargetFrame_CheckBattlePet(self);
		
		if ( self.petBattleIcon ) then
			self.petBattleIcon:SetAlpha(1.0);
		end
	end
end



function ALUF_TargetFrame_CheckFaction(self)

	if ( self.showPVP ) then
		local factionGroup = UnitFactionGroup(self.unit);
		if ( UnitIsPVPFreeForAll(self.unit) ) then
			self.PVPIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-FFA");
			self.PVPIcon:Show();
		elseif ( factionGroup and factionGroup ~= "Neutral" and UnitIsPVP(self.unit) ) then
			self.PVPIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..factionGroup);
			self.PVPIcon:Show();
		else
			self.PVPIcon:Hide();
		end
	end
	
end



function ALUF_TargetFrame_CheckClassification(self)

	local classification = UnitClassification(self.unit);
	
	-- TODO: Frametexture for "minus" and "rare" (low prio).
	if ( classification == "worldboss" or classification == "elite" or classification == "rareelite" ) then
		self.border:SetTexture("Interface\\AddOns\\\ArenaLiveUnitFrames\\Textures\\TargetingFrameElite");
	else
		self.border:SetTexture("Interface\\AddOns\\\ArenaLiveUnitFrames\\Textures\\TargetingFrame");
	end

end



function ALUF_TargetFrame_OnUpdate (self, elapsed)

	self.elapsed = (self.elapsed or 0) + elapsed;
	if ( self.elapsed > 0.5 ) then
		self.elapsed = 0;
		ArenaLive_ThreatIndicator_Update(self, self.feedbackUnit);
	end
end



function ALUF_TargetFrame_AdjustAttachments(self)
	
	local castBarIsShown, castBarPosition, castHistoryIsShown, auraFrameIsShown, auraFramePosition;
	local isEnabled = ARENALIVE_DATABASE[addonName]["frameTypes"][self.frameType]["enabled"];
	
	-- First check if the Frame is enabled. If not we can skip the rest.
	ArenaLive_ToggleBlizzFrame(TargetFrame, isEnabled);
	
	if ( isEnabled ) then
		ArenaLive_UnitFrame_UpdateUnit(self, "target");
		TargetFrame:SetScript("OnEvent", nil);
		ComboFrame:SetScript("OnEvent", nil);
	else
		ArenaLive_UnitFrame_ResetFrame(self);
		ArenaLive_CastBar_ResetFrame(ALUF_TargetFrameCastBar);
		ArenaLive_AuraFrame_ResetFrame(ALUF_TargetFrameAuras);
		ArenaLive_CastHistory_ResetFrame(self.castHistory);
		self:Hide();
		TargetFrame:SetScript("OnEvent", TargetFrame_OnEvent);
		ComboFrame:SetScript("OnEvent", ComboFrame_OnEvent);
		return;
	end
	

	
	if ( self.castBar ) then
		castBarIsShown = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["castBar"]["isShown"];
		castBarPosition = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["castBar"]["position"];
	end
	
	if ( self.castHistory ) then
		castHistoryIsShown = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["castHistory"]["isShown"];
	end
	
	if ( self.auraFrame ) then
		auraFrameIsShown = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["auraFrame"]["isShown"];
		auraFramePosition = ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["auraFrame"]["position"];
	end
	
	if ( castBarIsShown ) then
		ArenaLive_CastBar_UpdateUnit(ALUF_TargetFrameCastBar, "target");
		self.castBar:ClearAllPoints();
		
		if ( castBarPosition == auraFramePosition and auraFrameIsShown ) then
			--[[ castBar and auraFrame are either both above or below the unitframe. In this case we must anchor the castbar
			to the auraFrame, so that it will change it's position according to the height of the auraFrame. ]]--
			
			if ( castBarPosition == "BELOW" ) then
				self.castBar:SetPoint("TOPRIGHT", self.auraFrame, "BOTTOMRIGHT", -21, -3)
			elseif ( castBarPosition == "ABOVE" ) then
				self.castBar:SetPoint("BOTTOMRIGHT", self.auraFrame, "TOPRIGHT", -21, 3)
			end
		else
			--[[ Castbar and  auraFrame are on different spots, so the castbar needs to be anchored to the unitFrame itself]]--
			
			if ( castBarPosition == "BELOW" ) then
				self.castBar:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 10, -3)
			elseif ( castBarPosition == "ABOVE" ) then
				self.castBar:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 10, 3)
			end
		end
	else
		ArenaLive_CastBar_ResetFrame(ALUF_TargetFrameCastBar);
	end
	
	if ( castHistoryIsShown and castBarIsShown ) then
		-- We only show castHistory in combination with castBar at the moment.
		self.castHistory:ClearAllPoints();
		ArenaLive_CastHistory_UpdateUnit(ALUF_TargetFrameCastHistory, "target");
		if ( castBarPosition == "BELOW" ) then
			self.castHistory:SetPoint("TOPRIGHT", self.castBar.border, "BOTTOMRIGHT", 0, -2);
		elseif ( castBarPosition == "ABOVE" ) then
			self.castHistory:SetPoint("BOTTOMRIGHT", self.castBar.border, "TOPRIGHT", 0, 2);
		end
	else
		ArenaLive_CastHistory_ResetFrame(ALUF_TargetFrameCastHistory)
	end
	
	if ( auraFrameIsShown ) then
		self.auraFrame:ClearAllPoints();
		ArenaLive_AuraFrame_UpdateUnit(ALUF_TargetFrameAuras, "target");
		
		if ( auraFramePosition == "BELOW" ) then
			self.auraFrame:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", -33, 3);
			ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["auraFrame"]["growUpwards"] = false;
		elseif ( auraFramePosition == "ABOVE" ) then
			self.auraFrame:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", -33, 2);
			ARENALIVE_DATABASE["ArenaLiveUnitFrames"]["frameTypes"]["targetFrame"]["auraFrame"]["growUpwards"] = true;
		end
	else
		ArenaLive_AuraFrame_ResetFrame(ALUF_TargetFrameAuras)
	end
end